@@ -28,6 +28,7 @@
 ;; Feature masks:
 (define droppable-feature 1)
 (define scores-disabled 2)
+(define dealable-feature 4)
 
 (define (reset-features) (set-feature-word! 0))
 
@@ -140,7 +141,7 @@
 (define (shuffle-deck)
   (let* ((vec (list->vector DECK))
 	 (len (vector-length vec)))
-    (set! DECK (shuffle-deck-helper vec 0 len))))
+    (set! DECK (shuffle-deck-helper vec '() 0 len))))
 					     
 ; The playing area is divided into slots, where cards can be placed.
 ; Each slot can hold any amount of cards.  The slots are identified 
@@ -296,6 +297,17 @@
 (define (empty-slot? slot-id)
   (null? (get-cards slot-id)))
 
+(define (any-slot-empty? slots)
+  (if (eq? slots '())
+      #f
+      (or (empty-slot? (car slots))
+          (any-slot-empty? (cdr slots)))))
+
+(define (find-empty-slot slots)
+  (if (empty-slot? (car slots))
+      (car slots)
+      (find-empty-slot (cdr slots))))
+
 ; Get the nth card from a slot. Returns #f if n is out of range.
 (define (get-nth-card slot-id n)
   (let ((cards (get-cards slot-id)))
@@ -491,13 +503,13 @@
       (cons (make-card value suit) 
 	    (make-standard-deck-list-ace-low (+ 1 value) suit))))
 
-(define (shuffle-deck-helper deck ref1 len)
+(define (shuffle-deck-helper deck result ref1 len)
   (if (zero? len)
-      '()
+      result
       (let* ((ref2 (+ ref1 (random len)))
 	     (val-at-ref2 (vector-ref deck ref2)))
 	(vector-set! deck ref2 (vector-ref deck ref1))
-	(cons val-at-ref2 (shuffle-deck-helper deck (+ ref1 1) (- len 1))))))
+	(shuffle-deck-helper deck (cons val-at-ref2 result) (+ ref1 1) (- len 1)))))
 
 (define (new-slot deck placement)
   (list #f deck placement))
@@ -522,6 +534,12 @@
 (define (nthcdr n lst)
   (if (zero? n) lst (nthcdr (+ -1 n) (cdr lst))))
 
+; guile library function I'm not sure I can rely on
+(define (list-head lst k)
+  (if (= k 0)
+      '()
+      (cons (car lst) (list-head (cdr lst) (- k 1)))))
+
 ;; INTERNAL procedures
 
 ; global variables
